# -*- coding: utf-8 -*-

from numpy import array, zeros

from mosqito.sq_metrics.speech_intelligibility.sii_ansi._band_procedure_data import _get_critical_band_data, _get_equal_critical_band_data, _get_octave_band_data, _get_third_octave_band_data
from mosqito.sq_metrics.speech_intelligibility.sii_ansi._speech_data import _get_critical_band_speech_data, _get_equal_critical_band_speech_data, _get_octave_band_speech_data, _get_third_octave_band_speech_data
from mosqito.sq_metrics.speech_intelligibility.sii_ansi._main_sii import _main_sii
from mosqito.sound_level_meter.spectrum import spectrum
from mosqito.sound_level_meter.freq_band_synthesis import freq_band_synthesis


def sii_ansi(noise, fs, method, speech_level, threshold=None):
    """Calculate speech intelligibility index

    This function computes SII values for a noise time signal according to ANSI S3.5 standard.

    Parameters
    ----------
    noise : array_like
        Noise time signal in [Pa].
    fs: float
        Sampling frequency of the input noise signal.
    method: {"critical", "equally_critical", "third_octave", "octave"}
        Type of frequency band to be used for the calculation. See § 3.4 of the standard.
    speech_level : {'normal', 'raised', 'loud', 'shout'}
        Speech level to assess, the corresponding speech spectrum defined in the standard is used for calculation.
    threshold : array_like or 'zwicker'
        Threshold of hearing [dB ref. 2e-5 Pa] with same size as the chosen method frequency axis, or 'zwicker' to use the standard threshold.
        Default to None sets the threshold to zeros on each frequency band.
        
    Returns
    -------
    sii: numpy.ndarray
        Overall SII value.
    specific_sii: numpy.ndarray
        Specific SII values along the frequency axis.
    freq_axis: numpy.ndarray
        Frequency axis corresponding to the chosen method.
        
    Examples
    --------
    .. plot::
       :include-source:

        >>> from mosqito.sq_metrics.speech_intelligibility import sii
        >>> import matplotlib.pyplot as plt
        >>> import numpy as np
        >>> fs=48000
        >>> d=0.2
        >>> dB=90
        >>> time = np.arange(0, d, 1/fs)
        >>> f = 50
        >>> stimulus = np.sin(2 * np.pi * f * time) * np.sin(np.pi * f * time) + np.sin(10 * np.pi * f * time) + np.sin(100 * np.pi * f * time)
        >>> rms = np.sqrt(np.mean(np.power(stimulus, 2)))
        >>> ampl = 0.00002 * np.power(10, dB / 20) / rms
        >>> stimulus = stimulus * ampl
        >>> SII, SII_spec, freq_axis = sii(stimulus, fs, method='critical', speech_level='normal')
        >>> plt.plot(freq_axis, SII_spec)
        >>> plt.xlabel("Frequency [Hz]")
        >>> plt.ylabel("Specific value ")
        >>> plt.title("Speech Intelligibility Index = " + f"{SII:.2f}")   
        
    """
    
    if (method!='critical') & (method!='equally_critical') & (method!='third_octave') & (method!='octave'):
        raise ValueError('Method should be within {"critical", "equally_critical", "third_octave", "octave"}.')
        
    if (speech_level!='normal') & (speech_level!='raised') & (speech_level!='loud') & (speech_level!='shout'):
        raise ValueError('Speech level should be within {"normal", "raised", "loud", "shout"} to use the corresponding standard data.')
    
    # Get standard speech spectrum
    if method == 'critical':
        speech_spectrum, speech_level = _get_critical_band_speech_data(speech_level)
        CENTER_FREQUENCIES, LOWER_FREQUENCIES, UPPER_FREQUENCIES, _, _, _ = _get_critical_band_data()
    elif method == 'equally_critical':
        speech_spectrum, speech_level = _get_equal_critical_band_speech_data(speech_level)
        CENTER_FREQUENCIES, LOWER_FREQUENCIES, UPPER_FREQUENCIES, _, _, _ = _get_equal_critical_band_data()
    elif method == 'third_octave':
        speech_spectrum, speech_level = _get_third_octave_band_speech_data(speech_level)
        CENTER_FREQUENCIES, LOWER_FREQUENCIES, UPPER_FREQUENCIES, _, _, _, _ = _get_third_octave_band_data()
    elif method == 'octave':
        speech_spectrum, speech_level = _get_octave_band_speech_data(speech_level)
        CENTER_FREQUENCIES, LOWER_FREQUENCIES, UPPER_FREQUENCIES, _, _, _, _, = _get_octave_band_data()
    
    # Compute noise spectrum in dB
    spec, freqs = spectrum(noise, fs, nfft="default", window="blackman", db=True)
    noise_spectrum, _ = freq_band_synthesis(spec, freqs, LOWER_FREQUENCIES, UPPER_FREQUENCIES)
                
    SII, SII_specific, freq_axis = _main_sii(method, speech_spectrum, noise_spectrum, threshold)    
    
    return SII, SII_specific, freq_axis



# -*- coding: utf-8 -*-

# Third party imports
import numpy as np

# Local application imports
from mosqito.sound_level_meter import noct_spectrum
from mosqito.sq_metrics.loudness.loudness_zwst._main_loudness import _main_loudness
from mosqito.sq_metrics.loudness.loudness_zwst._calc_slopes import _calc_slopes
from mosqito.utils.conversion import amp2db

# Optional package import
try:
    from SciDataTool import DataTime, DataLinspace, DataFreq
except ImportError:
    DataTime = None
    DataLinspace = None
    DataFreq = None


def loudness_zwst(signal, fs=None, field_type="free", is_sdt_output=False):
    """Zwicker-loudness calculation for stationary signals

    Calculates the acoustic loudness according to Zwicker method for
    stationary signals.
    Normatice reference:
        ISO 532:1975 (method B)
        DIN 45631:1991
        ISO 532-1:2017 (method 1)
    The code is based on BASIC program published in "Program for
    calculating loudness according to DIN 45631 (ISO 532B)", E.Zwicker
    and H.Fastl, J.A.S.J (E) 12, 1 (1991).
    Note that due to normative continuity, as defined in the
    preceeding standards, the method is in accordance with
    ISO 226:1987 equal loudness contours (instead of ISO 226:2003)

    Parameters
    ----------
    signal : numpy.array or DataTime object
        Signal time values [Pa]
    fs : float, optional
        Sampling frequency, can be omitted if the input is a DataTime
        object. Default to None
    field_type : str
        Type of soundfield corresponding to spec_third ("free" by
        default or "diffuse").
    is_sdt_output : Bool, optional
        If True, the outputs are returned as SciDataTool objects.
        Default to False

    Outputs
    -------
    N : float or numpy.array
        The overall loudness array [sones], size (Ntime,).
    N_specific : numpy.ndarray or DataFreq object
        The specific loudness array [sones/bark], size (Nbark, Ntime).
    bark_axis: numpy.array
        The Bark axis array, size (Nbark,).
    """

    # Manage SciDataTool input type
    if DataTime is not None and isinstance(signal, DataTime):
        time = signal.get_along("time")["time"]
        fs = 1 / (time[1] - time[0])
        signal = signal.get_along("time")[signal.symbol]

    # Compute third octave band spectrum
    spec_third, _ = noct_spectrum(signal, fs, fmin=24, fmax=12600)

    # Compute dB values
    spec_third = amp2db(spec_third, ref=2e-5)

    # Compute main loudness
    Nm = _main_loudness(spec_third, field_type)

    # Computation of specific loudness pattern and integration of overall
    # loudness by attaching slopes towards higher frequencies
    N, N_specific = _calc_slopes(Nm)

    # Define Bark axis
    bark_axis = np.linspace(0.1, 24, int(24 / 0.1))

    # Manage SciDataTool output type
    if is_sdt_output:
        if DataLinspace is None:
            raise RuntimeError(
                "In order to handle Data objects you need the 'SciDataTool' package."
            )
        else:
            bark_data = DataLinspace(
                name="Critical band rate",
                unit="Bark",
                initial=0,
                final=24,
                number=int(24 / 0.1),
                include_endpoint=True,
            )
            N_specific = DataFreq(
                name="Specific loudness (Zwicker method for stationnary signal)",
                symbol="N'_{zwst}",
                axes=[bark_data],
                values=N_specific,
                unit="sone/Bark",
            )

    return N, N_specific, bark_axis



# -*- coding: utf-8 -*-
"""
@author: Daniel Jiménez-Caminero Costa
"""

import numpy as np


def sone2phone(sones):
    """ Conversion between sones and phones, based on the work done by Hugo Fastl and Eberhard Zwicker in the article
    "Psychoacoustics: Facts and Models".

    Parameters
    ----------
    sones: float
        One of the units in which loudness is measured. 1 sone equals to 40 phones. The conversion from sones to phones
        is not linear.

    Returns
    -------
    phons: float
        Logarithmic unit of loudness (phon = Ln). The dB SPL values of the signal at 1 kHz have the same value in
        phons. For example, 50 db SPL @ 1 kHz corresponds to 50 phones. The rest of the frequencies does not have
        the same co-relation between each other.

    """

    if sones >= 1:
        phons = 40 + ((10 * np.log10(sones)) / np.log10(2))

    else:
        phons = 40 * ((sones + 0.0005) ** 0.35)
    return phons



# -*- coding: utf-8 -*-

# Standard library imports
import numpy as np

from mosqito.sq_metrics.loudness.loudness_zwst._get_rns_index import _get_rns_index


def _calc_slopes(nm):
    """"""
    # Upper limits of approximated critical bands in terms of critical
    # band rate
    zup = np.array(
        [
            0.9,
            1.8,
            2.8,
            3.5,
            4.4,
            5.4,
            6.6,
            7.9,
            9.2,
            10.6,
            12.3,
            13.8,
            15.2,
            16.7,
            18.1,
            19.3,
            20.6,
            21.8,
            22.7,
            23.6,
            24,
        ]
    )
    # Range of specific loudness for the determination of the steepness
    # of the upper slopes in the specific loudness - critical band rate
    # pattern
    rns = np.array(
        [
            21.5,
            18,
            15.1,
            11.5,
            9,
            6.1,
            4.4,
            3.1,
            2.13,
            1.36,
            0.82,
            0.42,
            0.30,
            0.22,
            0.15,
            0.10,
            0.035,
            0,
        ]
    )
    # Steepness of the upper slopes in the specific loudness = Critical
    # band rate pattern for the ranges 'rns' as a function of the number
    # of the critical band
    usl = np.array(
        [
            (13, 8.2, 6.3, 5.5, 5.5, 5.5, 5.5, 5.5),
            (9, 7.5, 6, 5.1, 4.5, 4.5, 4.5, 4.5),
            (7.8, 6.7, 5.6, 4.9, 4.4, 3.9, 3.9, 3.9),
            (6.2, 5.4, 4.6, 4.0, 3.5, 3.2, 3.2, 3.2),
            (4.5, 3.8, 3.6, 3.2, 2.9, 2.7, 2.7, 2.7),
            (3.7, 3.0, 2.8, 2.35, 2.2, 2.2, 2.2, 2.2),
            (2.9, 2.3, 2.1, 1.9, 1.8, 1.7, 1.7, 1.7),
            (2.4, 1.7, 1.5, 1.35, 1.3, 1.3, 1.3, 1.3),
            (1.95, 1.45, 1.3, 1.15, 1.1, 1.1, 1.1, 1.1),
            (1.5, 1.2, 0.94, 0.86, 0.82, 0.82, 0.82, 0.82),
            (0.72, 0.67, 0.64, 0.63, 0.62, 0.62, 0.62, 0.62),
            (0.59, 0.53, 0.51, 0.50, 0.42, 0.42, 0.42, 0.42),
            (0.40, 0.33, 0.26, 0.24, 0.24, 0.22, 0.22, 0.22),
            (0.27, 0.21, 0.20, 0.18, 0.17, 0.17, 0.17, 0.17),
            (0.16, 0.15, 0.14, 0.12, 0.11, 0.11, 0.11, 0.11),
            (0.12, 0.11, 0.10, 0.08, 0.08, 0.08, 0.08, 0.08),
            (0.09, 0.08, 0.07, 0.06, 0.06, 0.06, 0.06, 0.05),
            (0.06, 0.05, 0.03, 0.02, 0.02, 0.02, 0.02, 0.02),
        ]
    )

    # From Ernesto Avedillo 13/feb/2022
    # Considering the original routine if ig > 7:ig = 7 until ig = 21 so I can append the last column until usl.shape = (18,21)
    usl_reshaped = np.append(usl, np.tile(
        usl[:, 7], 13).reshape(13, 18).T, axis=1)

    # For test, in case nm has only 1 col.
    len_1_nm = False
    if len(nm.shape) == 1:
        len_1_nm = True
        nm = np.append(nm, nm).reshape(2, 21).T

    # Start values
    data_length = nm.shape[1]
    nm_wide = nm.shape[0]
    spec_length = 240
    dec_compare = 8
    # Working array variables
    n2_array_specific = np.ones((spec_length, data_length))
    z2_array_specific = np.ones((spec_length, data_length))
    usl_array_specific = np.ones((spec_length, data_length))
    dz_array_specific = np.ones((spec_length, data_length))
    rns_values_specific = np.ones((spec_length, data_length))
    # Create a zup vector called zup_ea to define the position in the N_spezific array.
    zup_ea = (np.copy(zup) * 10).astype(np.int32)
    zup_ea = np.append(zup_ea, 0)

    # Create a complete array of ZUP vectors with shape nm elements in a row
    zup_array_ea = (np.ones((nm_wide, data_length)).T * zup).T
    # crearte an array of z values 0.1 increase

    # Prepare the array N_specific for output
    # For all cases where nm[:,col-1] < nm[:,col]
    N_specific = np.zeros((spec_length, data_length))
    # I save the first values for first raws defined in zup_ea
    # N_specific[:zup_ea[0],:] = np.multiply(N_specific[:zup_ea[0],:] , nm[1])
    # I complete the rest of the matrix extending the array 21  to 240
    for i in np.arange(nm_wide - 1):
        N_specific[zup_ea[i - 1]: zup_ea[i], :] = nm[i]

    N = np.zeros(data_length)

    # nm_aux = np.copy(nm)
    # nm_aux [0] = nm[1]

    # obtain the rns indexes for each value of the nm matrix.
    # rns_ind = _get_rns_index (nm, rns ,equal_too = True )
    rns_ind = _get_rns_index(nm, rns)
    # save in an array the values corresponding to this index
    rns_values = rns[rns_ind]

    # search usl_array for each nm cell
    usl_array_ind = np.array(
        [rns_ind.T, np.ones((zup.shape[0], data_length)).T * np.arange(21)], dtype=int
    ).transpose(1, 2, 0)
    usl_array = usl_reshaped[usl_array_ind[:, :, 0], usl_array_ind[:, :, 1]].T
    # create a 240 x nm shape[1] array to save the dz values equal to z2-z1
    dz = np.zeros((nm_wide, data_length))
    dz = zup_array_ea - np.roll(zup_array_ea, 1, axis=0)

    dz[0, :] = zup[1]
    dz[1, :] = zup[1]

    for i in np.arange(nm_wide):
        n2_array_specific[zup_ea[i - 1]: zup_ea[i], :] = nm[i]
        dz_array_specific[zup_ea[i - 1]: zup_ea[i], :] = dz[i]
        z2_array_specific[zup_ea[i - 1]: zup_ea[i], :] = zup_array_ea[i]
        usl_array_specific[zup_ea[i - 1]: zup_ea[i], :] = usl_array[i]
        rns_values_specific[zup_ea[i - 1]: zup_ea[i], :] = rns_values[i]

    j = 1
    n1_aux = np.zeros(data_length)
    z1_aux = np.zeros(data_length)

    for i in np.arange(nm_wide):
        # for all cases where nm[i-1]>nm[i]
        j = zup_ea[i - 1]
        indexes = _get_rns_index(n2_array_specific[j - 1], rns)
        rns_values_specific[j] = rns[indexes]
        usl_array_specific[j] = usl_reshaped[indexes, i - 1]

        mask_n1_bigger_nm = np.round(n2_array_specific[j - 1], dec_compare) > np.round(
            nm[i], dec_compare
        )
        # For all n1 <= nm[i] calculate (N = N + n2 * (z2 - z1))
        N[np.logical_not(mask_n1_bigger_nm)] += (
            n2_array_specific[j] * (z2_array_specific[j] - z1_aux)
        )[np.logical_not(mask_n1_bigger_nm)]
        n1_aux[np.logical_not(mask_n1_bigger_nm)] = np.copy(n2_array_specific[j])[
            np.logical_not(mask_n1_bigger_nm)
        ]
        z1_aux[np.logical_not(mask_n1_bigger_nm)] = np.copy(z2_array_specific[j])[
            np.logical_not(mask_n1_bigger_nm)
        ]

        if mask_n1_bigger_nm.sum() > 0:
            # For all n1 > nm[i] calculate z2,dz n2.
            # This routine
            # n2 = rns[j]
            # if n2 < nm[i]:
            #     n2 = nm[i]
            # dz = (n1 - n2) / usl[j, ig]
            # z2 = z1 + dz
            # if z2 > zup[i]:
            #     z2 = zup[i]
            #     dz = z2 - z1
            #     n2 = n1 - dz * usl[j, ig]
            # Can be subtituted by
            # max_rns_nm = max(rns[j],nm[i])
            # z2_ea = min((n1-max_rns_nm)/usl[j,ig]+z1,zup[i])
            # dz_ea = z2_ea - z1
            # n2_ea = n1 - dz_ea * usl[j,ig]

            Max_rns_nm_Array = np.maximum(rns_values_specific[j - 1], nm[i])
            z2_array_specific[j, mask_n1_bigger_nm] = np.minimum(
                np.divide(n1_aux - Max_rns_nm_Array,
                          usl_array_specific[j]) + z1_aux,
                zup[i],
            )[mask_n1_bigger_nm]
            dz_array_specific[j, mask_n1_bigger_nm] = (z2_array_specific[j] - z1_aux)[
                mask_n1_bigger_nm
            ]
            n2_array_specific[j, mask_n1_bigger_nm] = (
                n1_aux -
                np.multiply(dz_array_specific[j], usl_array_specific[j])
            )[mask_n1_bigger_nm]

            # Calculate N for all n1 > nm[i]

            N[mask_n1_bigger_nm] += (
                dz_array_specific[j] * (n1_aux + n2_array_specific[j]) / 2
            )[mask_n1_bigger_nm]

            # get value of z
            z_array = np.ones(data_length) * zup[i - 1] + 0.1

            for j in np.arange(zup_ea[i - 1], zup_ea[i]):

                # Save values from previous calculation (only after second loop)
                if j != zup_ea[i - 1]:
                    z2_array_specific[j, mask_n1_bigger_nm] = z2_array_specific[
                        j - 1, mask_n1_bigger_nm
                    ]
                    n2_array_specific[j, mask_n1_bigger_nm] = n2_array_specific[
                        j - 1, mask_n1_bigger_nm
                    ]
                    dz_array_specific[j, mask_n1_bigger_nm] = dz_array_specific[
                        j - 1, mask_n1_bigger_nm
                    ]
                    usl_array_specific[j, mask_n1_bigger_nm] = usl_array_specific[
                        j - 1, mask_n1_bigger_nm
                    ]
                    rns_values_specific[j, mask_n1_bigger_nm] = rns_values_specific[
                        j - 1, mask_n1_bigger_nm
                    ]

                # Sometimes a second loop is necesary if z > z2
                mask_z_bigger_z2 = np.logical_and(
                    mask_n1_bigger_nm,
                    np.round(z2_array_specific[j], dec_compare)
                    <= np.round(z_array, dec_compare),
                )
                if mask_z_bigger_z2.sum() > 0:
                    indexes = _get_rns_index(
                        n2_array_specific[j,
                                          mask_z_bigger_z2], rns, equal_too=True
                    )
                    rns_values_specific[j, mask_z_bigger_z2] = rns[indexes]
                    usl_array_specific[j, mask_z_bigger_z2] = usl_reshaped[
                        indexes, i - 1
                    ]
                    # Save Copy de Z2 y N2
                    n1_aux[mask_z_bigger_z2] = n2_array_specific[j,
                                                                 mask_z_bigger_z2]
                    z1_aux[mask_z_bigger_z2] = z2_array_specific[j,
                                                                 mask_z_bigger_z2]

                    # Vuelvo al inicio del loop y reviso de nuevo que n2 sea menor que nm
                    # Vuelvo a calcular mask_n1_bigger_nm
                    # Para los valores de n1_aux<=nm[i]
                    mask_z_bigger_z2_1 = np.logical_and(
                        mask_z_bigger_z2,
                        np.round(n1_aux, dec_compare) <= np.round(
                            nm[i], dec_compare),
                    )
                    n2_array_specific[j, mask_z_bigger_z2_1] = nm[i,
                                                                  mask_z_bigger_z2_1]
                    z2_array_specific[j, mask_z_bigger_z2_1] = zup[i]
                    dz_array_specific[j, mask_z_bigger_z2_1] = (
                        z2_array_specific[j] - z1_aux
                    )[mask_z_bigger_z2_1]
                    # Recalculate N
                    N[mask_z_bigger_z2_1] += (
                        n2_array_specific[j] * (z2_array_specific[j] - z1_aux)
                    )[mask_z_bigger_z2_1]

                    # For Values n1_aux>nm[i] after second loop
                    mask_z_bigger_z2_2 = np.logical_and(
                        mask_z_bigger_z2,
                        np.round(n1_aux, dec_compare) > np.round(
                            nm[i], dec_compare),
                    )
                    Max_rns_nm_Array = np.maximum(
                        rns_values_specific[j], nm[i])
                    z2_array_specific[j, mask_z_bigger_z2_2] = np.minimum(
                        np.divide(n1_aux - Max_rns_nm_Array,
                                  usl_array_specific[j])
                        + z1_aux,
                        zup[i],
                    )[mask_z_bigger_z2_2]
                    dz_array_specific[j, mask_z_bigger_z2_2] = (
                        z2_array_specific[j] - z1_aux
                    )[mask_z_bigger_z2_2]
                    n2_array_specific[j, mask_z_bigger_z2_2] = (
                        n1_aux
                        - np.multiply(dz_array_specific[j], usl_array_specific[j])
                    )[mask_z_bigger_z2_2]
                    # Recalculate N
                    N[mask_z_bigger_z2_2] += (
                        dz_array_specific[j] *
                        (n1_aux + n2_array_specific[j]) / 2
                    )[mask_z_bigger_z2_2]
                    N_specific[j, mask_z_bigger_z2_2] = (
                        n1_aux - np.multiply((z_array - z1_aux),
                                             usl_array_specific[j])
                    )[mask_z_bigger_z2_2]

                    # For the rest of the values  Where z < z2 calculate N_Specific
                    mask_z_rest = np.logical_xor(
                        mask_z_bigger_z2, mask_n1_bigger_nm)
                    N_specific[j, mask_z_rest] = (
                        n1_aux - np.multiply((z_array - z1_aux),
                                             usl_array_specific[j])
                    )[mask_z_rest]
                    z_array += 0.1

                    # Generate a new mask for n1 > nm[i]
                    mask_n1_bigger_nm = np.logical_xor(
                        mask_n1_bigger_nm, mask_z_bigger_z2_1
                    )

                else:
                    N_specific[j, mask_n1_bigger_nm] = (
                        n1_aux - np.multiply((z_array - z1_aux),
                                             usl_array_specific[j])
                    )[mask_n1_bigger_nm]
                    z_array += 0.1

                z1_aux = np.copy(z2_array_specific[j])
                n1_aux = np.copy(n2_array_specific[j])

                if mask_n1_bigger_nm.sum() == 0:
                    break
            z1_aux = np.copy(z2_array_specific[zup_ea[i] - 1])
            n1_aux = np.copy(n2_array_specific[zup_ea[i] - 1])

            indexes = _get_rns_index(
                n2_array_specific[j, mask_z_bigger_z2], rns, equal_too=True
            )
            rns_values_specific[j, mask_z_bigger_z2] = rns[indexes]
            usl_array_specific[j,
                               mask_z_bigger_z2] = usl_reshaped[indexes, i - 1]

    N[N < 0] = 0
    N[N <= 16] = np.floor(N[N <= 16] * 1000 + 0.5) / 1000
    N[N > 16] = np.floor(N[N > 16] * 100 + 0.5) / 100
    if len_1_nm:
        N = N[0]
        N_specific = N_specific[:, 0]

    return N, N_specific



# -*- coding: utf-8 -*-

# External import
import numpy as np

# Local imports
from mosqito.sq_metrics import loudness_zwtv
from mosqito.sq_metrics.sharpness.sharpness_din.sharpness_din_from_loudness import sharpness_din_from_loudness


def sharpness_din_tv(signal, fs, weighting="din", field_type="free", skip=0):
    """Acoustic sharpness calculation according to different methods
        (Aures, Von Bismarck, DIN 45692, Fastl) for a time varying signal.

    Parameters:
    ----------
    signal : numpy.array
        A time signal values [Pa], size (Ntime,) 
    fs : integer
        Sampling frequency.
    weighting : string
        To specify the weighting function used for the
        sharpness computation.'din' by default,'aures', 'bismarck','fastl'
    field_type : str
        Type of soundfield corresponding to spec_third ("free" by
        default or "diffuse").
    skip : float
        Number of second to be cut at the beginning of the analysis to skip the transient effect.

    Outputs
    ------
    S : float
        Sharpness value, size (Ntime,) .
    time_axis: numpy.array
        The time axis array, size (Ntime,) .

    """

    if skip == 0:
        print("[Warning] when computing sharpness from time-varying loudness, a transient effect appears on the first points. To cut it, use 'skip='")

    # Compute loudness
    N, N_specific, _, time_axis = loudness_zwtv(signal, fs)

    # Compute sharpness from loudness
    S = sharpness_din_from_loudness(
        N, N_specific, weighting=weighting, skip=0)

    # Cut transient effect
    cut_index = np.argmin(np.abs(time_axis - skip))

    return S[cut_index:], time_axis[cut_index:]



# -*- coding: utf-8 -*-

# Standard library import
import numpy as np

# Local application imports
from mosqito.sound_level_meter.noct_spectrum._center_freq import _center_freq
from mosqito.sound_level_meter.noct_spectrum._filter_bandwidth import _filter_bandwidth
from mosqito.sound_level_meter.noct_spectrum._n_oct_freq_filter import _n_oct_freq_filter


def noct_synthesis(spectrum, freqs, fmin, fmax, n=3, G=10, fr=1000):
    """Adapt input spectrum to nth-octave band spectrum
    
    Convert the input spectrum to third-octave band spectrum
    between "fc_min" and "fc_max".
    Parameters
    ----------
    spectrum : numpy.ndarray
        amplitude rms of the one-sided spectrum of the signal, size (nperseg, nseg).
    freqs : list
        List of input frequency , size (nperseg) or (nperseg, nseg).
    fmin : float
        Min frequency band [Hz].
    fmax : float
        Max frequency band [Hz].
    n : int
        Number of bands pr octave.
    G : int
        System for specifying the exact geometric mean frequencies.
        Can be base 2 or base 10.
    fr : int
        Reference frequency. Shall be set to 1 kHz for audible frequency
        range, to 1 Hz for infrasonic range (f < 20 Hz) and to 1 MHz for
        ultrasonic range (f > 31.5 kHz).
    Outputs
    -------
    spec : numpy.ndarray
        Third octave band spectrum of signal sig [dB re.2e-5 Pa], size (nbands, nseg).
    fpref : numpy.ndarray
        Corresponding preferred third octave band center frequencies, size (nbands).
    """
    
    # Deduce sampling frequency
    fs = np.mean(freqs[1:] - freqs[:-1]) * 2 * len(spectrum)

    # Get filters center frequencies
    fc_vec, fpref = _center_freq(fmin=fmin, fmax=fmax, n=n, G=G, fr=fr)
    
    # Compute the filters bandwidth
    alpha_vec, f_low, f_high = _filter_bandwidth(fc_vec, n=n)
    
    # Delete ends frequencies to prevent aliasing
    idx = np.asarray(np.where(f_high > fs / 2))
    if any(idx[0]):
        fc_vec = np.delete(fc_vec, idx)
        f_low = np.delete(f_low, idx)
        f_high = np.delete(f_high, idx)

    # Number of nth bands
    nband = len(fc_vec)
    
    # Results array initialization
    if len(spectrum.shape) > 1:
        nseg = spectrum.shape[1]
        spec = np.zeros((nband, nseg))
        # If only one axis is given, it is used for all the spectra
        if len(freqs.shape) == 1:
            freqs = np.tile(freqs, (nseg, 1)).T
    else:
        nseg = 1
        spec = np.zeros((nband))

    # Calculation of the rms level of the signal in each band
    spec = []
    for fc, alpha in zip(fc_vec, alpha_vec):
        spec.append(_n_oct_freq_filter(spectrum, fs, fc, alpha))

    return np.array(spec), fpref
